@@ -1,11 +1,11 @@
 import glob
-import json
 import os
 import platform
 import shutil
 import subprocess
 import sys
 import sysconfig
+from distutils import log
 from distutils.command.build import build as CommandBuild
 from distutils.errors import (
     CompileError,
@@ -14,14 +14,14 @@ from distutils.errors import (
     DistutilsPlatformError,
 )
 from distutils.sysconfig import get_config_var
-from subprocess import check_output
 from typing import Dict, List, NamedTuple, Optional, cast
 
 from setuptools.command.build_ext import build_ext as CommandBuildExt
 from setuptools.command.build_ext import get_abi3_suffix
+from typing_extensions import Literal
 
 from .command import RustCommand
-from .extension import Binding, RustExtension, Strip
+from .extension import RustExtension, Strip
 from .utils import (
     PyLimitedApi,
     binding_features,
@@ -51,11 +51,11 @@ class build_rust(RustCommand):
             "t",
             "directory for temporary files (cargo 'target' directory) ",
         ),
-        ("target", None, "Build for the target triple"),
+        ("target=", None, "Build for the target triple"),
     ]
     boolean_options = ["inplace", "debug", "release", "qbuild"]
 
-    plat_name: Optional[str]
+    plat_name: Optional[str] = None
 
     def initialize_options(self) -> None:
         super().initialize_options()
@@ -92,6 +92,12 @@ class build_rust(RustCommand):
         universal2 = False
         if self.plat_name.startswith("macosx-") and arch_flags:
             universal2 = "x86_64" in arch_flags and "arm64" in arch_flags
+            if not universal2 and not self.target:
+                if "arm64" in arch_flags:
+                    self.target = "aarch64-apple-darwin"
+                elif "x86_64" in arch_flags:
+                    self.target = "x86_64-apple-darwin"
+
         if universal2:
             arm64_dylib_paths = self.build_extension(ext, "aarch64-apple-darwin")
             x86_64_dylib_paths = self.build_extension(ext, "x86_64-apple-darwin")
@@ -107,7 +113,7 @@ class build_rust(RustCommand):
         self.install_extension(ext, dylib_paths)
 
     def build_extension(
-        self, ext: RustExtension, forced_target_triple: Optional[str]
+        self, ext: RustExtension, forced_target_triple: Optional[str] = None
     ) -> List["_BuiltModule"]:
 
         target_info = self._detect_rust_target(forced_target_triple)
@@ -175,7 +181,18 @@ class build_rust(RustCommand):
             # Tell musl targets not to statically link libc. See
             # https://github.com/rust-lang/rust/issues/59302 for details.
             if rustc_cfgs.get("target_env") == "musl":
-                rustc_args.extend(["-C", "target-feature=-crt-static"])
+                # This must go in the env otherwise rustc will refuse to build
+                # the cdylib, see https://github.com/rust-lang/cargo/issues/10143
+                MUSL_FLAGS = "-C target-feature=-crt-static"
+                rustflags = env.get("RUSTFLAGS")
+                if rustflags is not None:
+                    env["RUSTFLAGS"] = f"{rustflags} {MUSL_FLAGS}"
+                else:
+                    env["RUSTFLAGS"] = MUSL_FLAGS
+
+                # Include this in the command-line anyway, so that when verbose
+                # logging enabled the user will see that this flag is in use.
+                rustc_args.extend(MUSL_FLAGS.split())
 
             command = [
                 self.cargo,
@@ -210,7 +227,18 @@ class build_rust(RustCommand):
         # Find the shared library that cargo hopefully produced and copy
         # it into the build directory as if it were produced by build_ext.
 
-        artifacts_dir = os.path.join(target_dir, "debug" if debug else "release")
+        profile = ext.get_cargo_profile()
+        if profile:
+            # https://doc.rust-lang.org/cargo/reference/profiles.html
+            if profile in {"dev", "test"}:
+                profile_dir = "debug"
+            elif profile == "bench":
+                profile_dir = "release"
+            else:
+                profile_dir = profile
+        else:
+            profile_dir = "debug" if debug else "release"
+        artifacts_dir = os.path.join(target_dir, profile_dir)
         dylib_paths = []
 
         if ext._uses_exec_binding():
@@ -287,6 +315,7 @@ class build_rust(RustCommand):
                 ext_path = self.get_dylib_ext_path(ext, module_name)
                 os.makedirs(os.path.dirname(ext_path), exist_ok=True)
 
+            log.info("Copying rust artifact from %s to %s", dylib_path, ext_path)
             shutil.copyfile(dylib_path, ext_path)
 
             if sys.platform != "win32" and not debug_build:
@@ -316,9 +345,7 @@ class build_rust(RustCommand):
 
         ext_path: str = build_ext.get_ext_fullpath(target_fname)
 
-        if (ext.py_limited_api == "auto" and self._py_limited_api()) or (
-            ext.py_limited_api
-        ):
+        if _is_py_limited_api(ext.py_limited_api, self._py_limited_api()):
             abi3_suffix = get_abi3_suffix()
             if abi3_suffix is not None:
                 so_ext = get_config_var("EXT_SUFFIX")
@@ -366,7 +393,7 @@ class build_rust(RustCommand):
             return cast(PyLimitedApi, bdist_wheel_command.py_limited_api)
 
     def _detect_rust_target(
-        self, forced_target_triple: Optional[str]
+        self, forced_target_triple: Optional[str] = None
     ) -> Optional["_TargetInfo"]:
         cross_compile_info = _detect_unix_cross_compile_info()
         if cross_compile_info is not None:
@@ -452,7 +479,9 @@ class build_rust(RustCommand):
             args.extend(["--target", target_triple])
 
         if release:
-            args.append("--release")
+            profile = ext.get_cargo_profile()
+            if not profile:
+                args.append("--release")
 
         if quiet:
             args.append("-q")
@@ -578,6 +607,12 @@ def _detect_unix_cross_compile_info() -> Optional["_CrossCompileInfo"]:
         # not *NIX, or not cross compiling
         return None
 
+    if "apple-darwin" in host_type and (build_type and "apple-darwin" in build_type):
+        # On macos and the build and host differ. This is probably an arm
+        # Python which was built on x86_64. Don't try to handle this for now.
+        # (See https://github.com/PyO3/setuptools-rust/issues/192)
+        return None
+
     stdlib = sysconfig.get_path("stdlib")
     assert stdlib is not None
     cross_lib = os.path.dirname(stdlib)
@@ -652,17 +687,35 @@ def _base_cargo_target_dir(ext: RustExtension) -> str:
     If --target is passed to cargo in the command line, the target directory
     will have the target appended as a child.
     """
-    metadata_command = [
-        "cargo",
-        "metadata",
-        "--manifest-path",
-        ext.path,
-        "--format-version",
-        "1",
-    ]
-    metadata = json.loads(check_output(metadata_command))
-    target_directory = metadata["target_directory"]
+    target_directory = ext._metadata()["target_directory"]
     assert isinstance(
         target_directory, str
-    ), "expected cargo metadata to return a string target directory"
+    ), "expected cargo metadata to contain a string target directory"
     return target_directory
+
+
+def _is_py_limited_api(
+    ext_setting: Literal["auto", True, False],
+    wheel_setting: Optional[PyLimitedApi],
+) -> bool:
+    """Returns whether this extension is being built for the limited api.
+
+    >>> _is_py_limited_api("auto", None)
+    False
+
+    >>> _is_py_limited_api("auto", True)
+    True
+
+    >>> _is_py_limited_api(True, False)
+    True
+
+    >>> _is_py_limited_api(False, True)
+    False
+    """
+
+    # If the extension explicitly states to use py_limited_api or not, use that.
+    if ext_setting != "auto":
+        return ext_setting
+
+    # "auto" setting - use whether the bdist_wheel option is truthy.
+    return bool(wheel_setting)
